/*
	PlastererApplet
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "plastererimpl.h"
#include "plastererdefine.h"

CPlastererImpl::CPlastererImpl( QWidget* parent )
	: QWidget( parent, NULL )
{
	setWallPaper();

	m_pChannel = new QCopChannel( "QPE/Plasterer", this );
	connect( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	m_pSysChannel = new QCopChannel( "QPE/System", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveSysMessage( const QCString&, const QByteArray& ) ) );

	setFixedSize( 0, 0 );
}

CPlastererImpl::~CPlastererImpl()
{
	disconnect ( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	delete m_pChannel;
	delete m_pSysChannel;
}

void CPlastererImpl::receiveMessage( const QCString& msg, const QByteArray& /*data*/ )
{
	// QDataStream stream( data, IO_ReadOnly );

	if ( msg == "version()")
	{
		version();
	}
	else if ( msg == "reload()" )
	{
		DBG_LOG( "CPlasterer::receiveMessage() > \"reload()\"" );
		setWallPaper();
		QMessageBox mb( "Plasterer", "reloaded...", QMessageBox::NoIcon,
						QMessageBox::NoButton, QMessageBox::NoButton, QMessageBox::NoButton );
		mb.showMaximized();
	}
}

void CPlastererImpl::receiveSysMessage( const QCString& msg, const QByteArray& data )
{
	if ( msg == "rotateChanged(QString)" )
	{
		QDataStream stream( data, IO_ReadOnly );
		QString rotInf;
		stream >> rotInf;
		DBG_LOG( QString( "CPlastererImpl::receiveSysMessage() > data : %1" ).arg( rotInf ).latin1() );
		setWallPaper( ( rotInf == "Transformed:Rot270:Vga:0" ) ? WF_H_IMG : WF_V_IMG );
	}
}

void CPlastererImpl::version()
{
	QString ver = "Plasterer";
	ver.append( _version_ );
	Global::statusMessage( ver );
}

void CPlastererImpl::setWallPaper( const QString& sImg )
{
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	if ( sImg == QString::null )
	{
		sImgPath += SlMisc::isLandscape() ? WF_H_IMG : WF_V_IMG;
	}
	else
	{
		sImgPath += sImg;
	}
	Config cfg_l( CNF_L );
	Config cfg_p( CNF_P );
	cfg_l.setGroup( SEC_WP );
	cfg_l.writeEntry( KEY_WFN, sImgPath );
	DBG_LOG( QString( "CPlastererImpl::setWallPaper() > ImgPath : %1" ).arg( sImgPath ).latin1() );
	cfg_l.writeEntry( KEY_LBL, ( sImg == WF_V_IMG ) ? cfg_p.readNumEntry( KEY_VLB, 0 )
													: cfg_p.readNumEntry( KEY_HLB, 0 ) );
	
	{
		QCopEnvelope e( "QPE/System", "changedWallPaper()" );
	}
}

