/*
	PlastererUtil
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "PlastererUtil.h"

#include <stdlib.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qfontdatabase.h>
#include <qwhatsthis.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qpe/config.h>
#include <qpe/qcopenvelope_qws.h>
#include <qtimer.h>
#include <sl/sldlgwait.h>

#include "../applet/plastererdefine.h"
#include <sl/slfileselector.h>
#include <sl/slcolorselector.h>

////////////////////////////////////////////////
//    Image Browser
////////////////////////////////////////////////
ImageBrowser::ImageBrowser( QWidget* parent, 
							const char* name, bool modal, WFlags fl )
: QDialog( parent, name, modal, fl )
{
	setWState( WState_Reserved1 );

	if ( !name )
	{
		setName( "ImageBrowser" );
	}
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5,
												sizePolicy().hasHeightForWidth() ) );
	QString caption( tr("Select a Image") );
	setCaption( caption );

	BaseLayout = new QVBoxLayout( this ); 
	BaseLayout->setSpacing( 6 );
	BaseLayout->setMargin( 11 );

	ExeBrowser = new SlFileSelector( QPEApplication::documentDir(), "image/*", this, "ExeBrowser" );
	ExeBrowser->setFocusMode( SlFileSelector::ByTab );
	BaseLayout->addWidget( ExeBrowser );

	connect( ExeBrowser, SIGNAL( keyPressed( QKeyEvent*,bool& ) ),
				this, SLOT( BrowserKeyPressed( QKeyEvent*, bool& ) ) );

	showMaximized();
	ExeBrowser->setSelected();
}

ImageBrowser::~ImageBrowser()
{
}

void ImageBrowser::closeEvent( QCloseEvent *e )
{
	e->accept();
}

void ImageBrowser::accept()
{
	m_sPath = ExeBrowser->currentFile();
	if ( m_sPath[0] == '.' )
	{
		m_sPath = m_sPath.mid( 1 );
	}
	QFileInfo fi( m_sPath );
	if ( fi.isFile() )
	{
		emit done( QDialog::Accepted );
	}
	close();
}

void ImageBrowser::reject()
{
	close();
}

QString ImageBrowser::getPath()
{
	return m_sPath;
}

////////////////////////////////////////////////
//    Boot Message Setting Dialog
////////////////////////////////////////////////
BootMessageSetting::BootMessageSetting( LPBOOT_MESSAGE bootMessage,
									QWidget* parent = 0, const char* name = 0,
									bool modal = FALSE, WFlags fl = 0 )
: QDialog( parent, name, modal, fl )
, m_pBootMessage( bootMessage )
{
	setWState( WState_Reserved1 );
	if ( !name )
	{
		setName( "BootMessageSetting" );
	}
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5,
												sizePolicy().hasHeightForWidth() ) );
	QString caption( tr("Boot Message Settings") );
	setCaption( caption );

	m_tComboValue[ 0 ].nValue = BMPOS_CC;
	m_tComboValue[ 0 ].sLabel = tr( "X:CENTER Y:CENTER" );
	m_tComboValue[ 1 ].nValue = BMPOS_CL;
	m_tComboValue[ 1 ].sLabel = tr( "X:LEFT   Y:CENTER" );
	m_tComboValue[ 2 ].nValue = BMPOS_CR;
	m_tComboValue[ 2 ].sLabel = tr( "X:RIGHT  Y:CENTER" );
	m_tComboValue[ 3 ].nValue = BMPOS_TC;
	m_tComboValue[ 3 ].sLabel = tr( "X:CENTER Y:TOP   " );
	m_tComboValue[ 4 ].nValue = BMPOS_TL;
	m_tComboValue[ 4 ].sLabel = tr( "X:LEFT   Y:TOP   " );
	m_tComboValue[ 5 ].nValue = BMPOS_TR;
	m_tComboValue[ 5 ].sLabel = tr( "X:RIGHT  Y:TOP   " );
	m_tComboValue[ 6 ].nValue = BMPOS_BC;
	m_tComboValue[ 6 ].sLabel = tr( "X:CENTER Y:BOTTOM" );
	m_tComboValue[ 7 ].nValue = BMPOS_BL;
	m_tComboValue[ 7 ].sLabel = tr( "X:LEFT   Y:BOTTOM" );
	m_tComboValue[ 8 ].nValue = BMPOS_BR;
	m_tComboValue[ 8 ].sLabel = tr( "X:RIGHT  Y:BOTTOM" );

	BaseLayout = new QVBoxLayout( this );
	BaseLayout->setSpacing( 6 );
	BaseLayout->setMargin( 11 );
	BaseLayout->setAlignment( Qt::AlignTop );

	BootMessageLabel = new QLabel( tr("DisplayMessage :"), this, "BootMessageLabel" );
	BaseLayout->addWidget( BootMessageLabel );

	BootMessageEdit = new QLineEdit( this, "BootMessageEdit" );
	BaseLayout->addWidget( BootMessageEdit );

	FontFamilyLayout = new QHBoxLayout( this );
	FontFamilyLayout->setSpacing( 0 );
	FontFamilyLayout->setMargin( 0 );

	FontFamilyLabel = new QLabel( tr("Font Family :"), this );
	FontFamilyLayout->addWidget( FontFamilyLabel );

	FontFamilyCombo = new QComboBox( FALSE, this, "FontFamilyCombo" );
	FontFamilyLayout->addWidget( FontFamilyCombo );

	BaseLayout->addLayout( FontFamilyLayout );
	
	FontSizeLayout = new QHBoxLayout( this );
	FontSizeLayout->setSpacing( 0 );
	FontSizeLayout->setMargin( 0 );

	FontSizeLabel = new QLabel( tr("Font Size :"), this );
	FontSizeLayout->addWidget( FontSizeLabel );

	FontSizeCombo = new QComboBox( FALSE, this, "FontSizeCombo" );
	FontSizeLayout->addWidget( FontSizeCombo );

	BaseLayout->addLayout( FontSizeLayout );

	QFontMetrics fm( font() );
	FontColorButton = new QPushButton( this, "FontColorButton" );
	QString strButtonCaption( tr("Font Color") );
	QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
	tmpPixmap.fill( QColor(0,0,0) );
	QPainter p( &tmpPixmap );
	p.setPen( QColor(255,255,255) );
	p.drawText( 9, fm.height() + 2, strButtonCaption );
	p.flush();
	FontColorButton->setPixmap( tmpPixmap );

	BaseLayout->addWidget( FontColorButton );

	MessagePosLayout = new QHBoxLayout( this );
	MessagePosLayout->setSpacing( 0 );
	MessagePosLayout->setMargin( 0 );

	MessagePosLabel = new QLabel( tr("Display Pos. :"), this );
	MessagePosLayout->addWidget( MessagePosLabel );

	MessagePosCombo = new QComboBox( FALSE, this, "MessagePosCombo" );
	MessagePosLayout->addWidget( MessagePosCombo );

	BaseLayout->addLayout( MessagePosLayout );

	QStringList tmpList = QStringList::split( ",", m_pBootMessage->color );
	selectedFontColor( qRgb( tmpList[0].toInt(),
									tmpList[1].toInt(),
									tmpList[2].toInt() ) );

	BootMessageEdit->setText( m_pBootMessage->message );

	QFontDatabase fdb;

	QStringList availableFonts = fdb.families();
	for ( unsigned int nCnt = 0; nCnt < availableFonts.count(); nCnt ++ )
	{
		FontFamilyCombo->insertItem( availableFonts[ nCnt ], nCnt );
	}
	for ( unsigned int nCnt = 0; nCnt < availableFonts.count(); nCnt ++ )
	{
		if ( availableFonts[ nCnt ] == m_pBootMessage->font.family() )
		{
			FontFamilyCombo->setCurrentItem( nCnt );
		}
	}

	fontFamilyChanged( FontFamilyCombo->currentText() );
	for ( int nCnt = 0; nCnt < FontSizeCombo->count(); nCnt ++ )
	{
		QString tmpStr = FontSizeCombo->text( nCnt );
		if ( tmpStr.toInt() == m_pBootMessage->font.pointSize() )
		{
			FontSizeCombo->setCurrentItem( nCnt );
		}
	}

	for ( int nCnt = 0; nCnt < 9; nCnt ++ )
	{
		MessagePosCombo->insertItem( m_tComboValue[ nCnt ].sLabel, nCnt );
	}
	for ( int nCnt = 0; nCnt < 9; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].nValue == m_pBootMessage->position )
		{
			MessagePosCombo->setCurrentItem( nCnt );
		}
	}

	connect( FontColorButton, SIGNAL( clicked() ), this, SLOT( clickedFontColorButton() ) );
	connect( FontFamilyCombo, SIGNAL( activated(const QString&) ),
	                this, SLOT( fontFamilyChanged(const QString&) ) );
	showMaximized();
}

BootMessageSetting::~BootMessageSetting()
{
}

void BootMessageSetting::closeEvent( QCloseEvent *e )
{
	e->accept();
}

void BootMessageSetting::accept()
{
	m_pBootMessage->message = BootMessageEdit->text();
	QFont font( FontFamilyCombo->currentText(), FontSizeCombo->currentText().toInt() );
	m_pBootMessage->font = font;
	int pos = BMPOS_CC;
	for ( int nCnt = 0; nCnt < 9; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].sLabel == MessagePosCombo->currentText() )
		{
			pos = m_tComboValue[ nCnt ].nValue;
		}
	}
	m_pBootMessage->position = pos;
	QString tmpStr;
	m_pBootMessage->color = tmpStr.sprintf( "%d,%d,%d", m_clrFont.red(), m_clrFont.green(), m_clrFont.blue() );
	close();
}

void BootMessageSetting::reject()
{
	close();
}

void BootMessageSetting::fontFamilyChanged( const QString& family )
{
	QFontDatabase fdb;
	QValueList<int> availableSize = fdb.pointSizes( family );
	FontSizeCombo->clear();
	for ( unsigned int nCnt = 0; nCnt < availableSize.count(); nCnt ++ )
	{
		QString insStr;
		insStr = insStr.sprintf( "%d", availableSize[ nCnt ] );
		FontSizeCombo->insertItem( insStr, nCnt );
	}
}

void BootMessageSetting::clickedFontColorButton()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedFontColor( QRgb ) ) );
	QPoint tPoint( width() / 2 - 20, FontColorButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void BootMessageSetting::selectedFontColor( QRgb rgb )
{
	QColor clrT(rgb);
		QFontMetrics fm( font() );
		QString strButtonCaption( tr("Back Color") );
		QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
		tmpPixmap.fill( clrT );
		QPainter p( &tmpPixmap );
		p.setPen( QColor ( 0xFF & ~clrT.red(),
							0xFF & ~clrT.green(),
							0xFF & ~clrT.blue() ) );
		p.drawText( 9, fm.height() + 2, strButtonCaption );
		p.flush();
	FontColorButton->setPixmap( tmpPixmap );
	m_clrFont = clrT;
}

////////////////////////////////////////////////
//    Plasterer Util ( Main )
////////////////////////////////////////////////
PlastererUtil::PlastererUtil()
:PlastererUtilBase()
{
	m_pWaitDlg = NULL;
	m_bAccepted = FALSE;
	m_tComboValue[ 0 ].nValue = ILV_B;
	m_tComboValue[ 0 ].sLabel = tr( "Black" );
	m_tComboValue[ 1 ].nValue = ILV_W;
	m_tComboValue[ 1 ].sLabel = tr( "White" );
	m_tComboValue[ 2 ].nValue = ILV_BBW;
	m_tComboValue[ 2 ].sLabel = tr( "Black(WhiteBack)" );
	m_tComboValue[ 3 ].nValue = ILV_WWS;
	m_tComboValue[ 3 ].sLabel = tr( "White(WithShadow)" );
	m_tComboValue[ 4 ].nValue = ILV_WWR;
	m_tComboValue[ 4 ].sLabel = tr( "White(WithRim)" );
	m_tComboValue[ 5 ].nValue = ILV_NON;
	m_tComboValue[ 5 ].sLabel = tr( "None" );

	setWState( WState_Reserved1 );
	QString caption;
	caption = tr( "Plasterer" );
	caption += " Ver.";
	caption += _version_;
	setCaption( caption );

	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	ImagePathVEdit->setText( cfg.readEntry( KEY_VFN, "" ) );
	ImagePathHEdit->setText( cfg.readEntry( KEY_HFN, "" ) );
	ImagePathBootEdit->setText( cfg.readEntry( KEY_BFN, "" ) );
	m_strVFN = ImagePathVEdit->text();
	m_strHFN = ImagePathHEdit->text();
	m_strBFN = ImagePathBootEdit->text();

	ScalingVCheck->setChecked( cfg.readBoolEntry( KEY_VFT, FALSE ) );
	ScalingHCheck->setChecked( cfg.readBoolEntry( KEY_HFT, FALSE ) );
	ScalingBootCheck->setChecked( cfg.readBoolEntry( KEY_BFT, FALSE ) );
	m_bVFT = ScalingVCheck->isChecked();
	m_bHFT = ScalingHCheck->isChecked();
	m_bBFT = ScalingBootCheck->isChecked();

	m_strVBC = cfg.readEntry( KEY_VBC, "0,0,0" );
	{
		QStringList tmpList = QStringList::split( ",", m_strVBC );
		selectedBackColorV( qRgb( tmpList[0].toInt(),
									tmpList[1].toInt(),
									tmpList[2].toInt() ) );
	}
	
	m_strHBC = cfg.readEntry( KEY_HBC, "0,0,0" );
	{
		QStringList tmpList = QStringList::split( ",", m_strHBC );
		selectedBackColorH( qRgb( tmpList[0].toInt(),
									tmpList[1].toInt(),
									tmpList[2].toInt() ) );
	}

	m_strBBC = cfg.readEntry( KEY_BBC, "0,0,0" );
	{
		QStringList tmpList = QStringList::split( ",", m_strBBC );
		selectedBackColorBoot( qRgb( tmpList[0].toInt(),
									tmpList[1].toInt(),
									tmpList[2].toInt() ) );
	}

	if ( cfg.readBoolEntry( KEY_VTM, TRUE ) )
	{
		m_bVTM = TRUE;
		TilingVRadio->setChecked( TRUE );
		CenteringVRadio->setChecked( FALSE );
		ScalingVCheck->setChecked( FALSE );
		ScalingVCheck->setEnabled( FALSE );
		BackColorVButton->setEnabled( FALSE );
	}
	else
	{
		m_bVTM = FALSE;
		TilingVRadio->setChecked( FALSE );
		CenteringVRadio->setChecked( TRUE );
	}

	if ( cfg.readBoolEntry( KEY_HTM, TRUE ) )
	{
		m_bHTM = TRUE;
		TilingHRadio->setChecked( TRUE );
		CenteringHRadio->setChecked( FALSE );
		ScalingHCheck->setChecked( FALSE );
		ScalingHCheck->setEnabled( FALSE );
		BackColorHButton->setEnabled( FALSE );
	}
	else
	{
		m_bHTM = FALSE;
		TilingHRadio->setChecked( FALSE );
		CenteringHRadio->setChecked( TRUE );
	}

	if ( cfg.readBoolEntry( KEY_BTM, TRUE ) )
	{
		m_bBTM = TRUE;
		TilingBootRadio->setChecked( TRUE );
		CenteringBootRadio->setChecked( FALSE );
		ScalingBootCheck->setChecked( FALSE );
		ScalingBootCheck->setEnabled( FALSE );
		BackColorBootButton->setEnabled( FALSE );
	}
	else
	{
		m_bBTM = FALSE;
		TilingBootRadio->setChecked( FALSE );
		CenteringBootRadio->setChecked( TRUE );
	}

	if ( cfg.readBoolEntry( KEY_SVH, TRUE ) )
	{
		m_bSVH = TRUE;
		UseSameImageVHCheck->setChecked( TRUE );
		Tabs->setTabEnabled( TabH, FALSE );
	}
	else
	{
		m_bSVH = FALSE;
		UseSameImageVHCheck->setChecked( FALSE );
		Tabs->setTabEnabled( TabH, TRUE );
	}
	
	if ( cfg.readBoolEntry( KEY_DBI, TRUE ) )
	{
		m_bDBI = TRUE;
		UseDefaultImageCheck->setChecked( TRUE );
		BasicBootGroupBox->setEnabled( FALSE );
	}
	else
	{
		m_bDBI = FALSE;
		UseDefaultImageCheck->setChecked( FALSE );
		BasicBootGroupBox->setEnabled( TRUE );
	}
	
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		IconLabelVCombo->insertItem( m_tComboValue[ nCnt ].sLabel, nCnt );
		IconLabelHCombo->insertItem( m_tComboValue[ nCnt ].sLabel, nCnt );
	}
	m_nDefVLB = m_nVLB = cfg.readNumEntry( KEY_VLB, 0 );
	m_nDefHLB = m_nHLB = cfg.readNumEntry( KEY_HLB, 0 );
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].nValue == m_nDefVLB )
		{
			IconLabelVCombo->setCurrentItem( nCnt );
		}
	}
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].nValue == m_nDefHLB )
		{
			IconLabelHCombo->setCurrentItem( nCnt );
		}
	}

	{
		m_tDefBootMessage.message = cfg.readEntry( KEY_BDM, "" );
		m_tDefBootMessage.position = cfg.readNumEntry( KEY_MPS, BMPOS_CC );
		m_tDefBootMessage.color = cfg.readEntry( KEY_FCL, "0,0,0" );
		m_tDefBootMessage.font.setFamily( cfg.readEntry( KEY_FNT, "lcfont" ) );
		m_tDefBootMessage.font.setPointSize( cfg.readNumEntry( KEY_FSZ, 180 ) );

		m_tBootMessage.message = m_tDefBootMessage.message;
		m_tBootMessage.position = m_tDefBootMessage.position;
		m_tBootMessage.color = m_tDefBootMessage.color;
		m_tBootMessage.font = m_tDefBootMessage.font;
	}

	connect( ApplyButton, SIGNAL( clicked() ), this, SLOT( clickedApplyButton() ) );
	connect( OkButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( CancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( ImagePathVButton, SIGNAL( clicked() ), this, SLOT( clickedImagePathVButton() ) );
	connect( ImagePathHButton, SIGNAL( clicked() ), this, SLOT( clickedImagePathHButton() ) );
	connect( ImagePathBootButton, SIGNAL( clicked() ), this, SLOT( clickedImagePathBootButton() ) );
	connect( BackColorVButton, SIGNAL( clicked() ), this, SLOT( clickedBackColorVButton() ) );
	connect( BackColorHButton, SIGNAL( clicked() ), this, SLOT( clickedBackColorHButton() ) );
	connect( BackColorBootButton, SIGNAL( clicked() ), this, SLOT( clickedBackColorBootButton() ) );
	connect( BootMessageButton, SIGNAL( clicked() ), this, SLOT( clickedBootMessageButton() ) );
	connect( ImageAlignVGroup, SIGNAL( clicked( int ) ), 
						this, SLOT( clickedImageAlignVGroup( int ) ) );
	connect( ImageAlignHGroup, SIGNAL( clicked( int ) ), 
						this, SLOT( clickedImageAlignHGroup( int ) ) );
	connect( ImageAlignBootGroup, SIGNAL( clicked( int ) ), 
						this, SLOT( clickedImageAlignBootGroup( int ) ) );
	connect( UseSameImageVHCheck, SIGNAL( clicked() ), this, SLOT( clickedUseSameImageCheck() ) );
	connect( UseDefaultImageCheck, SIGNAL( clicked() ), this, SLOT( clickedUseDefaultImageCheck() ) );
	connect( IconLabelVCombo, SIGNAL( activated(const QString&) ),
	                this, SLOT( iconLabelVChanged(const QString&) ) );
	connect( IconLabelHCombo, SIGNAL( activated(const QString&) ),
	                this, SLOT( iconLabelHChanged(const QString&) ) );
}

PlastererUtil::~PlastererUtil()
{
}

void PlastererUtil::closeEvent( QCloseEvent *e )
{
	if ( ! m_bAccepted )
	{
		m_pWaitDlg = new SlDlgWait( this, "Plasterer\n now Making Images", TRUE, FALSE );
		m_pWaitDlg->move( QPoint( ( QPEApplication::desktop()->width() - m_pWaitDlg->width() ) / 2 ,
							( QPEApplication::desktop()->height() - m_pWaitDlg->height() ) / 2 ) );
		m_pWaitDlg->exec();

		Config cfg( CNF_P );
		cfg.setGroup( SEC_WP );

		cfg.writeEntry( KEY_VFN, m_strVFN );
		cfg.writeEntry( KEY_HFN, m_strHFN );
		cfg.writeEntry( KEY_BFN, m_strBFN );

		cfg.writeEntry( KEY_VFT, m_bVFT );
		cfg.writeEntry( KEY_HFT, m_bHFT );
		cfg.writeEntry( KEY_BFT, m_bBFT );

		cfg.writeEntry( KEY_VTM, m_bVTM );
		cfg.writeEntry( KEY_HTM, m_bHTM );
		cfg.writeEntry( KEY_BTM, m_bBTM );

		cfg.writeEntry( KEY_VBC, m_strVBC );
		cfg.writeEntry( KEY_HBC, m_strHBC );
		cfg.writeEntry( KEY_BBC, m_strBBC );

		cfg.writeEntry( KEY_VLB, m_nDefVLB );
		cfg.writeEntry( KEY_HLB, m_nDefHLB );

		cfg.writeEntry( KEY_SVH, m_bSVH );

		cfg.writeEntry( KEY_DBI, m_bDBI );

		cfg.writeEntry( KEY_BDM, m_tDefBootMessage.message );
		cfg.writeEntry( KEY_FNT, m_tDefBootMessage.font.family() );
		cfg.writeEntry( KEY_FSZ, m_tDefBootMessage.font.pointSize() );
		cfg.writeEntry( KEY_FCL, m_tDefBootMessage.color );
		cfg.writeEntry( KEY_MPS, m_tDefBootMessage.position );

		QTimer::singleShot( 1, this, SLOT( applyWallpaper() ) );
	}
	e->accept();
}

void PlastererUtil::accept()
{
	m_bAccepted = TRUE;
	clickedApplyButton();
	close();
}

void PlastererUtil::reject()
{
	close();
}

void PlastererUtil::clickedUseSameImageCheck()
{
	if ( UseSameImageVHCheck->isChecked() )
	{
		Tabs->setTabEnabled( TabH, FALSE );
	}
	else
	{
		Tabs->setTabEnabled( TabH, TRUE );
	}
}

void PlastererUtil::clickedUseDefaultImageCheck()
{
	if ( UseDefaultImageCheck->isChecked() )
	{
		BasicBootGroupBox->setEnabled( FALSE );
	}
	else
	{
		BasicBootGroupBox->setEnabled( TRUE );
	}
}

void PlastererUtil::clickedImageAlignVGroup( int id )
{
	if ( id == idTiling )
	{
		ScalingVCheck->setChecked( FALSE );
		ScalingVCheck->setEnabled( FALSE );
		BackColorVButton->setEnabled( FALSE );
	}
	else
	{
		ScalingVCheck->setEnabled( TRUE );
		BackColorVButton->setEnabled( TRUE );
	}
}

void PlastererUtil::clickedImageAlignHGroup( int id )
{
	if ( id == idTiling )
	{
		ScalingHCheck->setChecked( FALSE );
		ScalingHCheck->setEnabled( FALSE );
		BackColorHButton->setEnabled( FALSE );
	}
	else
	{
		ScalingHCheck->setEnabled( TRUE );
		BackColorHButton->setEnabled( TRUE );
	}
}

void PlastererUtil::clickedImageAlignBootGroup( int id )
{
	if ( id == idTiling )
	{
		ScalingBootCheck->setChecked( FALSE );
		ScalingBootCheck->setEnabled( FALSE );
		BackColorBootButton->setEnabled( FALSE );
	}
	else
	{
		ScalingBootCheck->setEnabled( TRUE );
		BackColorBootButton->setEnabled( TRUE );
	}
}

void PlastererUtil::iconLabelVChanged( const QString& str )
{
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].sLabel == str )
		{
			m_nVLB = m_tComboValue[ nCnt ].nValue;
			break;
		}
	}
}

void PlastererUtil::iconLabelHChanged( const QString& str )
{
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].sLabel == str )
		{
			m_nHLB = m_tComboValue[ nCnt ].nValue;
			break;
		}
	}
}

void PlastererUtil::clickedApplyButton()
{
	if ( m_pWaitDlg )
	{
		return;
	}
	m_pWaitDlg = new SlDlgWait( this, "Plasterer\n now Making Images", TRUE, FALSE );
	m_pWaitDlg->move( QPoint( ( QPEApplication::desktop()->width() - m_pWaitDlg->width() ) / 2 ,
						( QPEApplication::desktop()->height() - m_pWaitDlg->height() ) / 2 ) );
	m_pWaitDlg->exec();

	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	cfg.writeEntry( KEY_SVH, UseSameImageVHCheck->isChecked() );

	cfg.writeEntry( KEY_VFN, ImagePathVEdit->text() );
	cfg.writeEntry( KEY_VFT, ScalingVCheck->isChecked() );
	cfg.writeEntry( KEY_VTM, TilingVRadio->isChecked() );
	
	QString tmpStr;
	tmpStr.sprintf( "%d,%d,%d", m_clrVBC.red(), m_clrVBC.green(), m_clrVBC.blue() );
	cfg.writeEntry( KEY_VBC, tmpStr );
	cfg.writeEntry( KEY_VLB, m_nVLB );

	if ( UseSameImageVHCheck->isChecked() )
	{
		cfg.writeEntry( KEY_HFN, ImagePathVEdit->text() );
		cfg.writeEntry( KEY_HFT, ScalingVCheck->isChecked() );
		cfg.writeEntry( KEY_HTM, TilingVRadio->isChecked() );
		cfg.writeEntry( KEY_HBC, tmpStr );
		cfg.writeEntry( KEY_HLB, m_nVLB );
	}
	else
	{
		cfg.writeEntry( KEY_HFN, ImagePathHEdit->text() );
		cfg.writeEntry( KEY_HFT, ScalingHCheck->isChecked() );
		cfg.writeEntry( KEY_HTM, TilingHRadio->isChecked() );
		tmpStr.sprintf( "%d,%d,%d", m_clrHBC.red(), m_clrHBC.green(), m_clrHBC.blue() );
		cfg.writeEntry( KEY_HBC, tmpStr );
		cfg.writeEntry( KEY_HLB, m_nHLB );
	}

	cfg.writeEntry( KEY_DBI, UseDefaultImageCheck->isChecked() );
	cfg.writeEntry( KEY_BFN, ImagePathBootEdit->text() );
	cfg.writeEntry( KEY_BFT, ScalingBootCheck->isChecked() );
	cfg.writeEntry( KEY_BTM, TilingBootRadio->isChecked() );
	
	tmpStr.sprintf( "%d,%d,%d", m_clrBBC.red(), m_clrBBC.green(), m_clrBBC.blue() );
	cfg.writeEntry( KEY_BBC, tmpStr );

	cfg.writeEntry( KEY_BDM, m_tBootMessage.message );
	cfg.writeEntry( KEY_FNT, m_tBootMessage.font.family() );
	cfg.writeEntry( KEY_FSZ, m_tBootMessage.font.pointSize() );
	cfg.writeEntry( KEY_FCL, m_tBootMessage.color );
	cfg.writeEntry( KEY_MPS, m_tBootMessage.position );

	QTimer::singleShot( 1, this, SLOT( applyWallpaper() ) );
}

void PlastererUtil::applyWallpaper()
{
	makeImages();
	QCopEnvelope e( "QPE/Plasterer", "reload()" );
}

void PlastererUtil::clickedBackColorVButton()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedBackColorV( QRgb ) ) );
	QPoint tPoint( width() / 2 - 20, BackColorVButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void PlastererUtil::selectedBackColorV( QRgb rgb )
{
	QColor clrT(rgb);
		QFontMetrics fm( font() );
		QString strButtonCaption( tr("Back Color") );
		QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
		tmpPixmap.fill( clrT );
		QPainter p( &tmpPixmap );
		p.setPen( QColor ( 0xFF & ~clrT.red(),
							0xFF & ~clrT.green(),
							0xFF & ~clrT.blue() ) );
		p.drawText( 9, fm.height() + 2, strButtonCaption );
		p.flush();
	BackColorVButton->setPixmap( tmpPixmap );
	m_clrVBC = clrT;
}

void PlastererUtil::clickedBackColorHButton()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedBackColorH( QRgb ) ) );
	QPoint tPoint( width() / 2 - 20, BackColorHButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void PlastererUtil::selectedBackColorH( QRgb rgb )
{
	QColor clrT(rgb);
		QFontMetrics fm( font() );
		QString strButtonCaption( tr("Back Color") );
		QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
		tmpPixmap.fill( clrT );
		QPainter p( &tmpPixmap );
		p.setPen( QColor ( 0xFF & ~clrT.red(),
							0xFF & ~clrT.green(),
							0xFF & ~clrT.blue() ) );
		p.drawText( 9, fm.height() + 2, strButtonCaption );
		p.flush();
	BackColorHButton->setPixmap( tmpPixmap );
	m_clrHBC = clrT;
}

void PlastererUtil::clickedBackColorBootButton()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedBackColorBoot( QRgb ) ) );
	QPoint tPoint( width() / 2 - 20, BackColorHButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void PlastererUtil::selectedBackColorBoot( QRgb rgb )
{
	QColor clrT(rgb);
		QFontMetrics fm( font() );
		QString strButtonCaption( tr("Back Color") );
		QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
		tmpPixmap.fill( clrT );
		QPainter p( &tmpPixmap );
		p.setPen( QColor ( 0xFF & ~clrT.red(),
							0xFF & ~clrT.green(),
							0xFF & ~clrT.blue() ) );
		p.drawText( 9, fm.height() + 2, strButtonCaption );
		p.flush();
	BackColorBootButton->setPixmap( tmpPixmap );
	m_clrBBC = clrT;
}

void PlastererUtil::clickedImagePathVButton()
{
	ImageBrowser dlg( this, "Image Selector", TRUE, WType_Modal );
	if ( dlg.exec() == QDialog::Accepted )
	{
		ImagePathVEdit->setText( dlg.getPath() );
	}
}

void PlastererUtil::clickedImagePathHButton()
{
	ImageBrowser dlg( this, "Image Selector", TRUE, WType_Modal );
	if ( dlg.exec() == QDialog::Accepted )
	{
		ImagePathHEdit->setText( dlg.getPath() );
	}
}

void PlastererUtil::clickedImagePathBootButton()
{
	ImageBrowser dlg( this, "Image Selector", TRUE, WType_Modal );
	if ( dlg.exec() == QDialog::Accepted )
	{
		ImagePathBootEdit->setText( dlg.getPath() );
	}
}

void PlastererUtil::makeImages()
{
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	QSize maxWidgetSize = QPEApplication::desktop()->size();

	bool isVGA = ( QMAX( maxWidgetSize.width(), maxWidgetSize.height() ) > 320 ) ? TRUE : FALSE;
	QString pathBootImg = isVGA ? WF_BOOT_PATH_L : WF_BOOT_PATH_S;
	QString pathDefBootImg = isVGA ? WF_DEF_BOOT_L : WF_DEF_BOOT_S;
	QImage imgB( cfg.readEntry( KEY_BFN, "" ) );
	makeBootImage( &imgB, cfg, maxWidgetSize );
	rotateBootImg( &imgB );
	if ( imgB.size() == QSize( 0, 0 ) )
	{
		// use Default Boot Time Image
		if ( QFile::exists( sImgPath + WF_B_IMG ) )
		{
			if ( ! QFile::remove( sImgPath + WF_B_IMG ) )
			{
				DBG_LOG( QString("makeImages() > QFile::remove(\"%1\") function returned with ERROR")
									.arg( sImgPath + WF_B_IMG ).latin1() );
			}
		}

		DBG_LOG( QString( "/bin/ln -s -f %1 %2" )
									.arg( pathDefBootImg )
									.arg( pathBootImg + WF_B_IMG )
									.latin1() );
		int nRes = system( QString( "/bin/ln -s -f %1 %2" )
									.arg( pathDefBootImg )
									.arg( pathBootImg + WF_B_IMG )
									.latin1() );
		if ( nRes )
		{
			DBG_LOG( "makeImages() > ::system() function returned with ERROR" );
		}
	}
	else
	{
		// use Customized Boot Time Image
		imgB.save( sImgPath + WF_B_IMG, "BMP" );
		DBG_LOG( QString( "/bin/ln -s -f %1 %2" )
									.arg( sImgPath + WF_B_IMG )
									.arg( pathBootImg + WF_B_IMG )
									.latin1() );
		int nRes = system( QString( "/bin/ln -s -f %1 %2" )
									.arg( sImgPath + WF_B_IMG )
									.arg( pathBootImg + WF_B_IMG )
									.latin1() );
		if ( nRes )
		{
			DBG_LOG( "makeImages() > ::system() function returned with ERROR" );
		}
	}

	if ( maxWidgetSize.height() < maxWidgetSize.width() )
	{
		maxWidgetSize = QSize( maxWidgetSize.height(), maxWidgetSize.width() );
	}

	QSize sizeWidget;

	int corWidth = cfg.readNumEntry( KEY_WCO, -1 );
	if ( corWidth < 0 )
	{
		corWidth = SlMisc::getScrollbarMetrics().width();
	}
	int corHeight = cfg.readNumEntry( KEY_HCO, -1 );

	sizeWidget = QSize( maxWidgetSize.width() - corWidth,
						corHeight >= 0 ? maxWidgetSize.height() - corHeight
										: (int) ( maxWidgetSize.height() * 0.80 ) );
	QImage imgV( cfg.readEntry( KEY_VFN, "" ) );
	makeImage( &imgV, cfg, sizeWidget, KEY_VFN );
	imgV.save( sImgPath + WF_V_IMG, "PNG" );

	sizeWidget = QSize( corHeight >= 0 ? maxWidgetSize.width() - corHeight
										: (int) ( maxWidgetSize.width() * 0.80 ),
						maxWidgetSize.height() - corWidth );
	QImage imgH( cfg.readEntry( KEY_HFN, "" ) );
	makeImage( &imgH, cfg, sizeWidget, KEY_HFN );
	imgH.save( sImgPath + WF_H_IMG, "PNG" );

	if ( m_pWaitDlg )
	{
		delete m_pWaitDlg;
		m_pWaitDlg = NULL;
	}
}

void PlastererUtil::makeImage( QImage* img,
								Config& cfg,
								const QSize& rSizeDesktop,
								const QString keyFName )
{
	bool isAlignV = ( keyFName == KEY_VFN ) ? TRUE : FALSE;

	if ( ! cfg.readBoolEntry( isAlignV ? KEY_VTM : KEY_HTM, TRUE )
		&& ( ! img->isNull() )
		&& ( img->size() != QSize( 0, 0 ) ) )
	{
		QSize rSize;
		if ( rSizeDesktop.height() > rSizeDesktop.width() )
		{
			rSize = isAlignV ? rSizeDesktop : QSize( rSizeDesktop.height(), rSizeDesktop.width() );
		}
		else
		{
			rSize = isAlignV ? QSize( rSizeDesktop.height(), rSizeDesktop.width() ) : rSizeDesktop;
		}
		QSize sizeScaling = img->size();

		if ( cfg.readBoolEntry( isAlignV ? KEY_VFT : KEY_HFT, FALSE ) )
		{
			if ( isAlignV )
			{
				sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)rSize.height() / (double)sizeScaling.height() ) );
				sizeScaling.setHeight( rSize.height() );

				sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)rSize.width() / (double)sizeScaling.width() ) );
				sizeScaling.setWidth( rSize.width() );
			}
			else
			{
				sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)rSize.width() / (double)sizeScaling.width() ) );
				sizeScaling.setWidth( rSize.width() );

				sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)rSize.height() / (double)sizeScaling.height() ) );
				sizeScaling.setHeight( rSize.height() );
			}

			if ( img->size() != sizeScaling )
			{
				*img = img->smoothScale( sizeScaling.width(), sizeScaling.height() );
			}
		}

		QPoint drawP( ( rSize.width() - sizeScaling.width() ) / 2,
							( rSize.height() - sizeScaling.height() ) / 2 );
		QPixmap pixT;
		pixT.resize( rSize );
		QStringList tmpList = QStringList::split( ",", cfg.readEntry( isAlignV ? KEY_VBC : KEY_HBC, "0,0,0" ) );
		QColor tmpColor( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
		pixT.fill( tmpColor );
		QPainter p( &pixT );
		p.drawImage( drawP, *img );
		p.flush();
		*img = pixT.convertToImage();
	}
}

void PlastererUtil::makeBootImage( QImage* img,
								Config& cfg,
								const QSize& rSizeDesktop )
{
	DBG_LOG( "makeBootImage() LOGIC START ------>" );

	DBG_LOG( QString( "makeBootImage() > DesktopSize : w%1/h%2" )
														.arg( rSizeDesktop.width() )
														.arg( rSizeDesktop.height() ).latin1() );
														
	bool isVGA = ( QMAX( rSizeDesktop.width(), rSizeDesktop.height() ) > 320 ) ? TRUE : FALSE;

	QPixmap pixT;

	if ( ( ! img->isNull() )
		&& ( img->size() != QSize( 0, 0 ) )
		&& ( ! cfg.readBoolEntry( KEY_DBI, TRUE ) ) )
	{
		DBG_LOG( "makeBootImage() > Make Customized Boot Time Wall Image" );

		//pixT.resize( rSizeDesktop.height(), rSizeDesktop.width() );
		{
			QString pathDefBootImg = isVGA ? WF_DEF_BOOT_L : WF_DEF_BOOT_S;
			QImage tmpImg( pathDefBootImg );
			pixT.resize( tmpImg.height(), tmpImg.width() );
		}
		bool isAlignV = ( pixT.height() > pixT.width() ) ? TRUE : FALSE;

		QPainter p( &pixT );
	
		if ( cfg.readBoolEntry( KEY_BTM, FALSE ) )
		{
			DBG_LOG( "makeBootImage() > TILING-MODE" );

			QSize sizeImg = img->size();
			DBG_LOG( QString( "makeBootImage() > Set Image Size : w%1/h%2" )
																.arg( sizeImg.width() )
																.arg( sizeImg.height() ).latin1() );

			int nX = pixT.width() / sizeImg.width() 
							+ ( ( pixT.width() % sizeImg.width() != 0 ) ? 1 : 0 );
			int nY = pixT.height() / sizeImg.height() 
							+ ( ( pixT.height() % sizeImg.height() != 0 ) ? 1 : 0 );
			DBG_LOG( QString( "makeBootImage() > will be draw x:%1 y:%2" ).arg( nX ).arg( nY ).latin1() );

			QPoint drawP( 0, 0 );
			DBG_LOG( QString( "makeBootImage() > Draw Point : x%1/y%2" )
															.arg( drawP.x() )
															.arg( drawP.y() ).latin1() );
			for ( int nXCnt = 0; nXCnt < nX; nXCnt ++ )
			{
			//	p.drawImage( drawP, *img );
				for ( int nYCnt = 0; nYCnt < nY; nYCnt ++ )
				{
					p.drawImage( drawP, *img );
					drawP.setY( drawP.y() + sizeImg.height() );
					DBG_LOG( QString( "makeBootImage() > Draw Point : x%1/y%2" )
																	.arg( drawP.x() )
																	.arg( drawP.y() ).latin1() );
				}
				drawP.setX( drawP.x() + sizeImg.width() );
				drawP.setY( 0 );
				DBG_LOG( QString( "makeBootImage() > Draw Point : x%1/y%2" )
																.arg( drawP.x() )
																.arg( drawP.y() ).latin1() );
			}
		}
		else
		{
			DBG_LOG( "makeBootImage() > CENTERING-MODE" );
			
			QStringList tmpList = QStringList::split( ",", cfg.readEntry( KEY_BBC, "0,0,0" ) );
			QColor tmpColor( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
			pixT.fill( tmpColor );

			QSize sizeScaling = img->size();
			DBG_LOG( QString( "makeBootImage() > Before Scaling Image Size : w%1/h%2" )
																.arg( sizeScaling.width() )
																.arg( sizeScaling.height() ).latin1() );
	
			if ( cfg.readBoolEntry( KEY_BFT, FALSE ) )
			{
				DBG_LOG( "makeBootImage() > FITTORESOLUTION-MODE" );

				if ( isAlignV )
				{
					sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)pixT.height() / (double)sizeScaling.height() ) );
					sizeScaling.setHeight( pixT.height() );
	
					sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)pixT.width() / (double)sizeScaling.width() ) );
					sizeScaling.setWidth( pixT.width() );
				}
				else
				{
					sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)pixT.width() / (double)sizeScaling.width() ) );
					sizeScaling.setWidth( pixT.width() );

					sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)pixT.height() / (double)sizeScaling.height() ) );
					sizeScaling.setHeight( pixT.height() );
				}
	
				DBG_LOG( QString( "makeBootImage() > Computed Size : w%1/h%2" )
																		.arg( sizeScaling.width() )
																		.arg( sizeScaling.height() )
																		.latin1() );
				if ( img->size() != sizeScaling )
				{
					*img = img->smoothScale( sizeScaling.width(), sizeScaling.height() );
				}
			}
	
			QPoint drawP( ( pixT.width() - sizeScaling.width() ) / 2,
								( pixT.height() - sizeScaling.height() ) / 2 );
			p.drawImage( drawP, *img );
		}

		QString message = cfg.readEntry( KEY_BDM, "" );
		if ( ! message.isEmpty() )
		{
			QFont font( cfg.readEntry( KEY_FNT, "lcfont" ), cfg.readNumEntry( KEY_FSZ, 160 ) / 10 );
			p.setFont( font );
			QStringList tmpList = QStringList::split( ",", cfg.readEntry( KEY_FCL, "0,0,0" ) );
			QColor color( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
			p.setPen( color );

			QFontMetrics fm( font );
			QSize sizeMessage( fm.width( message ), fm.height() );
			QPoint drawPos;

			int nBMPOS = cfg.readNumEntry( KEY_MPS, BMPOS_CC );
			switch ( nBMPOS )
			{
			case BMPOS_TR:
			case BMPOS_TC:
			case BMPOS_TL:
				{
					drawPos.setY( sizeMessage.height() );
				}
				break;
			case BMPOS_BR:
			case BMPOS_BC:
			case BMPOS_BL:
				{
					drawPos.setY( pixT.height() );
				}
				break;
			case BMPOS_CR:
			case BMPOS_CC:
			case BMPOS_CL:
			default:
				{
					drawPos.setY( (int)( ( pixT.height() + sizeMessage.height() ) / 2 ) );
				}
				break;
			}
			
			switch ( nBMPOS )
			{
			case BMPOS_TR:
			case BMPOS_CR:
			case BMPOS_BR:
				{
					drawPos.setX( pixT.width() - sizeMessage.width() );
				}
				break;
			case BMPOS_TL:
			case BMPOS_CL:
			case BMPOS_BL:
				{
					drawPos.setX( 0 );
				}
				break;
			case BMPOS_TC:
			case BMPOS_CC:
			case BMPOS_BC:
			default:
				{
					drawPos.setX( (int)( ( pixT.width() - sizeMessage.width() ) / 2 ) );
				}
				break;
			}
			p.drawText( drawPos, message );
		}

		p.flush();

		// pixT.resize( rSizeDesktop.height(), rSizeDesktop.width() );
	}
	else
	{
		DBG_LOG( "makeBootImage() > Nothing to do." );
		pixT.resize( 0, 0 );
	}
	
	*img = pixT.convertToImage();

	DBG_LOG( "makeBootImage() LOGIC END <------" );
}

void PlastererUtil::clickedBootMessageButton()
{
	BootMessageSetting dlg( &m_tBootMessage, this, "Boot Message Setting", TRUE, WType_Modal );
	if ( dlg.exec() == QDialog::Accepted )
	{
		// not write
	}
}

void PlastererUtil::rotateBootImg( QImage* img )
{
	QPixmap pixT;
	pixT.convertFromImage( *img );
	QWMatrix ma;
	ma.rotate( 90 );
	pixT = pixT.xForm( ma );
	*img = pixT.convertToImage();
}

