/*
	PlastererUtil
	Copyright (C) 2003 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __PLASTERER_UTIL_H__
#define __PLASTERER_UTIL_H__

#include <qvariant.h>
#include <qdialog.h>
#include "PlastererUtilBase.h"

class QVBoxLayout; 
class QHBoxLayout; 
class QGroupBox;
class QImage;
class Config;
class QLabel;
class QPushButton;
class SlFileSelector;
class SlColorSelector;
class SlDlgWait;

////////////////////////////////////////////
//    Image Browser
////////////////////////////////////////////
class ImageBrowser : public QDialog
{
Q_OBJECT

public:
	ImageBrowser( QWidget* parent = 0, const char* name = 0,
							bool modal = FALSE, WFlags fl = 0 );
	~ImageBrowser();
	QString getPath();

private:
	QVBoxLayout * BaseLayout;
	SlFileSelector * ExeBrowser;
	QString m_sPath;

protected:
	void closeEvent( QCloseEvent * );

protected slots:
	void accept();
	void reject();
};

////////////////////////////////////////////
//    Boot Message Structure
////////////////////////////////////////////
typedef struct tagBootMessage
{
	QString		message;
	QFont		font;
	QString		color;
	int			position;
} BOOT_MESSAGE, *LPBOOT_MESSAGE;

////////////////////////////////////////////
//    For ComboBox Structure
////////////////////////////////////////////
typedef struct tagComboSet
{
	int nValue;
	QString sLabel;
} LABEL_VALUE_MAP, *LPLABEL_VALUE_SET;

////////////////////////////////////////////
//    Boot Message Setting Dialog
////////////////////////////////////////////
class BootMessageSetting : public QDialog
{
Q_OBJECT

public:
	BootMessageSetting( LPBOOT_MESSAGE bootMessage,
									QWidget* parent = 0, const char* name = 0,
									bool modal = FALSE, WFlags fl = 0 );
	~BootMessageSetting();

	QLabel*			BootMessageLabel;
	QLineEdit*		BootMessageEdit;
	QLabel*			FontFamilyLabel;
	QComboBox*		FontFamilyCombo;
	QLabel*			FontSizeLabel;
	QComboBox*		FontSizeCombo;
	QPushButton*	FontColorButton;
	QLabel*			MessagePosLabel;
	QComboBox*		MessagePosCombo;
	
protected:
	void closeEvent( QCloseEvent * );

protected slots:
	void accept();
	void reject();
	void selectedFontColor( QRgb );
	void clickedFontColorButton();
	void fontFamilyChanged( const QString& );

protected:
	QVBoxLayout*	BaseLayout;
	QHBoxLayout*	FontFamilyLayout;
	QHBoxLayout*	FontSizeLayout;
	QHBoxLayout*	MessagePosLayout;
	LABEL_VALUE_MAP	m_tComboValue[9]; 
	LPBOOT_MESSAGE	m_pBootMessage;
	//BOOT_MESSAGE	m_tBootMessage;
	QColor			m_clrFont;
};

////////////////////////////////////////////
//    Plasterer Util ( Main )
////////////////////////////////////////////
class PlastererUtil : public PlastererUtilBase
{ 
Q_OBJECT

// non-slot public methods
public:
    PlastererUtil();
    ~PlastererUtil();

// slot methods
protected slots:
	void accept();
	void reject();
	void clickedUseSameImageCheck();
	void clickedUseDefaultImageCheck();
	void clickedImageAlignVGroup( int );
	void clickedImageAlignHGroup( int );
	void clickedImageAlignBootGroup( int );
	void clickedApplyButton();
	void clickedBackColorVButton();
	void clickedBackColorHButton();
	void clickedBackColorBootButton();
	void clickedImagePathVButton();
	void clickedImagePathHButton();
	void clickedImagePathBootButton();
	void clickedBootMessageButton();
	void selectedBackColorV( QRgb );
	void selectedBackColorH( QRgb );
	void selectedBackColorBoot( QRgb );
	void applyWallpaper();
	void iconLabelVChanged( const QString& );
	void iconLabelHChanged( const QString& );

// non-slot private methods
protected:
	void closeEvent( QCloseEvent* );
	void makeImages();
	void makeImage( QImage*, Config& , const QSize&, const QString );
	void makeBootImage( QImage*, Config& , const QSize& );
	void rotateBootImg( QImage * );

// menber valiables
protected:
	SlDlgWait*		m_pWaitDlg;
	QString			m_strVFN;
	QString			m_strHFN;
	QString			m_strBFN;
	bool 			m_bAccepted;
	bool 			m_bVFT;
	bool			m_bHFT;
	bool			m_bBFT;
	bool			m_bVTM;
	bool			m_bHTM;
	bool			m_bBTM;
	QString			m_strVBC;
	QString			m_strHBC;
	QString			m_strBBC;
	bool			m_bSVH;
	bool			m_bDBI;

	QColor			m_clrVBC;
	QColor			m_clrHBC;
	QColor			m_clrBBC;

	LABEL_VALUE_MAP	m_tComboValue[6]; 

	int				m_nVLB;
	int				m_nHLB;
	int				m_nDefVLB;
	int				m_nDefHLB;

	BOOT_MESSAGE 	m_tBootMessage;
	BOOT_MESSAGE 	m_tDefBootMessage;
};

#endif // __PLASTERER_UTIL_H__

