/*
	PlastererUtil
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "PlastererUtilBase.h"

#include <qtabwidget.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qwhatsthis.h>
#include <qpe/qpeapplication.h>
#include <qpe/resource.h>

PlastererUtilBase::PlastererUtilBase()
:QMainWindow()
{
	resize( 553, 263 ); 
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5,
												sizePolicy().hasHeightForWidth() ) );
	BaseFormLayout = new QVBoxLayout( this ); 
	BaseFormLayout->setSpacing( 6 );
	BaseFormLayout->setMargin( 11 );

	Tabs = new QTabWidget( this, "Tabs" );
	BaseFormLayout->addWidget( Tabs );

	// Settings for Vertical direction display ( VIEW-STYLE )
	{
		TabV = new QWidget( Tabs );
		TabBaseVLayout = new QVBoxLayout( TabV ); 
		TabBaseVLayout->setSpacing( 6 );
		TabBaseVLayout->setMargin( 11 );

		BasicVGroupBox = new QGroupBox( TabV, "BasicVGroupBox" );
		BasicVGroupBox->setTitle( "" );
		BasicVGroupBox->setColumnLayout(0, Qt::Vertical );
		BasicVGroupBox->layout()->setSpacing( 0 );
		BasicVGroupBox->layout()->setMargin( 0 );
		BasicGroupBoxVLayout = new QVBoxLayout( BasicVGroupBox->layout() );
		BasicGroupBoxVLayout->setAlignment( Qt::AlignTop );
		BasicGroupBoxVLayout->setSpacing( 6 );
		BasicGroupBoxVLayout->setMargin( 11 );

		UseSameImageVHCheck = new QCheckBox( tr( "Use Same Image for Input-style" ),
													TabV, "UseSameImageVHCheck" );

		// Image Path
		{
			ImagePathVLayout = new QHBoxLayout( BasicVGroupBox ); 
			ImagePathVLayout->setSpacing( 0 );
			ImagePathVLayout->setMargin( 0 );
	
			ImagePathVLabel = new QLabel( tr("Image Path :"), BasicVGroupBox, "ImagePathVLabel" );
			ImagePathVLayout->addWidget( ImagePathVLabel );

			ImagePathVEdit = new QLineEdit( BasicVGroupBox, "ImagePathVEdit" );
			ImagePathVLayout->addWidget( ImagePathVEdit );

			ImagePathVButton = new QPushButton( BasicVGroupBox, "ImagePathVButton" );
			ImagePathVButton->setText( "..." );
			ImagePathVLayout->addWidget( ImagePathVButton );
		}

		// Align Mode
		ImageAlignVGroup = new QButtonGroup( 2, Qt::Horizontal, BasicVGroupBox, "ImageAlignVGroup" );
		ImageAlignVGroup->setFrameShape( QFrame::NoFrame );
		{
			TilingVRadio = new QRadioButton( tr( "Tile" ), ImageAlignVGroup, "TilingVRadio" );
			ImageAlignVGroup->insert( TilingVRadio, idTiling );
			CenteringVRadio = new QRadioButton( tr( "Center" ), ImageAlignVGroup, "CenteringVRadio" );
			ImageAlignVGroup->insert( CenteringVRadio, idCentering );
		}

		ScalingVCheck = new QCheckBox( tr( "Scale" ), BasicVGroupBox, "ScalingVCheck" );

		// BackgroundColor
		{
			QFontMetrics fm( font() );
			BackColorVButton = new QPushButton( BasicVGroupBox, "BackColorVButton" );
			QString strButtonCaption( tr("Back Color") );
			QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
			tmpPixmap.fill( QColor(0,0,0) );
			QPainter p( &tmpPixmap );
			p.setPen( QColor(255,255,255) );
			p.drawText( 9, fm.height() + 2, strButtonCaption );
			p.flush();
			BackColorVButton->setPixmap( tmpPixmap );
		}

		// BGColor&ScaleCheck Horizontal Layout
		{
			ForCenteringVLayout = new QHBoxLayout( BasicVGroupBox ); 
			ForCenteringVLayout->setSpacing( 0 );
			ForCenteringVLayout->setMargin( 0 );

			ForCenteringVLayout->addWidget( ScalingVCheck );
			ForCenteringVLayout->addWidget( BackColorVButton );
		}
		
		// Icon Label
		{
			IconLabelVLayout = new QHBoxLayout( BasicVGroupBox ); 
			IconLabelVLayout->setSpacing( 0 );
			IconLabelVLayout->setMargin( 0 );
	
			IconLabelVLabel = new QLabel( tr("Label :"), BasicVGroupBox, "IconLabelVLabel" );
			IconLabelVLayout->addWidget( IconLabelVLabel );

			IconLabelVCombo = new QComboBox( FALSE, BasicVGroupBox, "IconLabelVCombo" );
			IconLabelVLayout->addWidget( IconLabelVCombo );
		}

		TabBaseVLayout->addWidget( UseSameImageVHCheck );
		// BasicGroupBoxVLayout->addWidget( ImagePathVLabel );
		BasicGroupBoxVLayout->addLayout( ImagePathVLayout );
		BasicGroupBoxVLayout->addWidget( ImageAlignVGroup );
		// BasicGroupBoxVLayout->addWidget( ScalingVCheck );
		// BasicGroupBoxVLayout->addWidget( BackColorVButton );
		BasicGroupBoxVLayout->addLayout( ForCenteringVLayout );
		BasicGroupBoxVLayout->addLayout( IconLabelVLayout );
		TabBaseVLayout->addWidget( BasicVGroupBox );
	}
	Tabs->addTab( TabV, tr( "VERTICAL" ) );

	// Settings for Horizontal direction display ( INPUT-STYLE )
	{
		TabH = new QWidget( Tabs );
		TabBaseHLayout = new QVBoxLayout( TabH ); 
		TabBaseHLayout->setSpacing( 6 );
		TabBaseHLayout->setMargin( 11 );

		BasicHGroupBox = new QGroupBox( TabH, "BasicHGroupBox" );
		BasicHGroupBox->setTitle( "" );
		BasicHGroupBox->setColumnLayout(0, Qt::Vertical );
		BasicHGroupBox->layout()->setSpacing( 0 );
		BasicHGroupBox->layout()->setMargin( 0 );
		BasicGroupBoxHLayout = new QVBoxLayout( BasicHGroupBox->layout() );
		BasicGroupBoxHLayout->setAlignment( Qt::AlignTop );
		BasicGroupBoxHLayout->setSpacing( 6 );
		BasicGroupBoxHLayout->setMargin( 11 );

		// Image Path
		{
			ImagePathHLayout = new QHBoxLayout( BasicHGroupBox ); 
			ImagePathHLayout->setSpacing( 0 );
			ImagePathHLayout->setMargin( 0 );
	
			ImagePathHLabel = new QLabel( tr("Image Path :"), BasicHGroupBox, "ImagePathHLabel" );
			ImagePathHLayout->addWidget( ImagePathHLabel );

			ImagePathHEdit = new QLineEdit( BasicHGroupBox, "ImagePathHEdit" );
			ImagePathHLayout->addWidget( ImagePathHEdit );

			ImagePathHButton = new QPushButton( BasicHGroupBox, "ImagePathHButton" );
			ImagePathHButton->setText( "..." );
			ImagePathHLayout->addWidget( ImagePathHButton );
		}

		// Align Mode
		ImageAlignHGroup = new QButtonGroup( 2, Qt::Horizontal, BasicHGroupBox, "ImageAlignHGroup" );
		ImageAlignHGroup->setFrameShape( QFrame::NoFrame );
		{
			TilingHRadio = new QRadioButton( tr( "Tile" ), ImageAlignHGroup, "TilingHRadio" );
			ImageAlignHGroup->insert( TilingHRadio, idTiling );
			CenteringHRadio = new QRadioButton( tr( "Center" ), ImageAlignHGroup, "CenteringHRadio" );
			ImageAlignHGroup->insert( CenteringHRadio, idCentering );
		}
			
		ScalingHCheck = new QCheckBox( tr( "Scale" ), BasicHGroupBox, "ScalingHCheck" );

		// BackgroundColor
		{
			QFontMetrics fm( font() );
			BackColorHButton = new QPushButton( BasicHGroupBox, "BackColorHButton" );
			QString strButtonCaption( tr("Back Color") );
			QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
			tmpPixmap.fill( QColor(0,0,0) );
			QPainter p( &tmpPixmap );
			p.setPen( QColor(255,255,255) );
			p.drawText( 9, fm.height() + 2, strButtonCaption );
			p.flush();
			BackColorHButton->setPixmap( tmpPixmap );
		}

		// BGColor&ScaleCheck Horizontal Layout
		{
			ForCenteringHLayout = new QHBoxLayout( BasicHGroupBox ); 
			ForCenteringHLayout->setSpacing( 0 );
			ForCenteringHLayout->setMargin( 0 );

			ForCenteringHLayout->addWidget( ScalingHCheck );
			ForCenteringHLayout->addWidget( BackColorHButton );
		}
		
		// Icon Label
		{
			IconLabelHLayout = new QHBoxLayout( BasicHGroupBox ); 
			IconLabelHLayout->setSpacing( 0 );
			IconLabelHLayout->setMargin( 0 );
	
			IconLabelHLabel = new QLabel( tr("Label :"), BasicHGroupBox, "IconLabelHLabel" );
			IconLabelHLayout->addWidget( IconLabelHLabel );

			IconLabelHCombo = new QComboBox( FALSE, BasicHGroupBox, "IconLabelHCombo" );
			IconLabelHLayout->addWidget( IconLabelHCombo );
		}

		// BasicGroupBoxHLayout->addWidget( ImagePathHLabel );
		BasicGroupBoxHLayout->addLayout( ImagePathHLayout );
		BasicGroupBoxHLayout->addWidget( ImageAlignHGroup );
		// BasicGroupBoxHLayout->addWidget( ScalingHCheck );
		// BasicGroupBoxHLayout->addWidget( BackColorHButton );
		BasicGroupBoxHLayout->addLayout( ForCenteringHLayout );
		BasicGroupBoxHLayout->addLayout( IconLabelHLayout );
		TabBaseHLayout->addWidget( BasicHGroupBox );
	}
	Tabs->addTab( TabH, tr( "HORIZONTAL" ) );

	// Settings for boot time display ( Wait a few minutes )
	{
		TabBoot = new QWidget( Tabs );
		TabBaseBootLayout = new QVBoxLayout( TabBoot ); 
		TabBaseBootLayout->setSpacing( 6 );
		TabBaseBootLayout->setMargin( 11 );

		BasicBootGroupBox = new QGroupBox( TabBoot, "BasicBootGroupBox" );
		BasicBootGroupBox->setTitle( "" );
		BasicBootGroupBox->setColumnLayout(0, Qt::Vertical );
		BasicBootGroupBox->layout()->setSpacing( 0 );
		BasicBootGroupBox->layout()->setMargin( 0 );
		BasicGroupBoxBootLayout = new QVBoxLayout( BasicBootGroupBox->layout() );
		BasicGroupBoxBootLayout->setAlignment( Qt::AlignTop );
		BasicGroupBoxBootLayout->setSpacing( 6 );
		BasicGroupBoxBootLayout->setMargin( 11 );

		UseDefaultImageCheck = new QCheckBox( tr( "Use Default Image" ),
													TabBoot, "UseDefaultImageCheck" );

		// Image Path
		{
			ImagePathBootLayout = new QHBoxLayout( BasicBootGroupBox ); 
			ImagePathBootLayout->setSpacing( 0 );
			ImagePathBootLayout->setMargin( 0 );
	
			ImagePathBootLabel = new QLabel( tr("Image Path :"),
												BasicBootGroupBox,
												"ImagePathBootLabel" );
			ImagePathBootLayout->addWidget( ImagePathBootLabel );

			ImagePathBootEdit = new QLineEdit( BasicBootGroupBox, "ImagePathBootEdit" );
			ImagePathBootLayout->addWidget( ImagePathBootEdit );

			ImagePathBootButton = new QPushButton( BasicBootGroupBox, "ImagePathBootButton" );
			ImagePathBootButton->setText( "..." );
			ImagePathBootLayout->addWidget( ImagePathBootButton );
		}

		// Align Mode
		ImageAlignBootGroup = new QButtonGroup( 2, Qt::Horizontal,
												BasicBootGroupBox, "ImageAlignBootGroup" );
		ImageAlignBootGroup->setFrameShape( QFrame::NoFrame );
		{
			TilingBootRadio = new QRadioButton( tr( "Tile" ), ImageAlignBootGroup,
																	"TilingBootRadio" );
			ImageAlignBootGroup->insert( TilingBootRadio, idTiling );
			CenteringBootRadio = new QRadioButton( tr( "Center" ), ImageAlignBootGroup,
																	"CenteringBootRadio" );
			ImageAlignBootGroup->insert( CenteringBootRadio, idCentering );
		}
			
		ScalingBootCheck = new QCheckBox( tr( "Scale" ), BasicBootGroupBox, "ScalingBootCheck" );

		// BackgroundColor
		{
			QFontMetrics fm( font() );
			BackColorBootButton = new QPushButton( BasicBootGroupBox, "BackColorBootButton" );
			QString strButtonCaption( tr("Back Color") );
			QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
			tmpPixmap.fill( QColor(0,0,0) );
			QPainter p( &tmpPixmap );
			p.setPen( QColor(255,255,255) );
			p.drawText( 9, fm.height() + 2, strButtonCaption );
			p.flush();
			BackColorBootButton->setPixmap( tmpPixmap );
		}

		// BGColor&ScaleCheck Horizontal Layout
		{
			ForCenteringBootLayout = new QHBoxLayout( BasicBootGroupBox ); 
			ForCenteringBootLayout->setSpacing( 0 );
			ForCenteringBootLayout->setMargin( 0 );

			ForCenteringBootLayout->addWidget( ScalingBootCheck );
			ForCenteringBootLayout->addWidget( BackColorBootButton );
		}

		BootMessageButton = new QPushButton( BasicBootGroupBox, "BootMessageButton" );
		BootMessageButton->setText( tr("Display Message") );

		BasicGroupBoxBootLayout->addLayout( ImagePathBootLayout );
		BasicGroupBoxBootLayout->addWidget( ImageAlignBootGroup );
		// BasicGroupBoxBootLayout->addWidget( ScalingBootCheck );
		// BasicGroupBoxBootLayout->addWidget( BackColorBootButton );
		BasicGroupBoxBootLayout->addLayout( ForCenteringBootLayout );
		BasicGroupBoxBootLayout->addWidget( BootMessageButton );
		TabBaseBootLayout->addWidget( UseDefaultImageCheck );
		TabBaseBootLayout->addWidget( BasicBootGroupBox );
	}
	Tabs->addTab( TabBoot, tr( "BootTime" ) );

	ButtonsLayout = new QHBoxLayout( this ); 
	ButtonsLayout->setSpacing( 0 );
	ButtonsLayout->setMargin( 0 );
	ApplyButton = new QPushButton( this, "ApplyButton" );
	ApplyButton->setText( tr("Apply") );
	OkButton = new QPushButton( this, "OkButton" );
	OkButton->setText( tr("OK") );
	CancelButton = new QPushButton( this, "CancelButton" );
	CancelButton->setText( tr("Cancel") );
	ButtonsLayout->addWidget( ApplyButton );
	ButtonsLayout->addWidget( OkButton );
	ButtonsLayout->addWidget( CancelButton );
	BaseFormLayout->addLayout( ButtonsLayout );
}

PlastererUtilBase::~PlastererUtilBase()
{
	// no need to delete child widgets, Qt does it all for us
}

